from typing import Union

from mermaid.graph import Graph
from mermaid.sequence.element import Actor, Box, Note, NotePosition, Participant, Rect
from mermaid.sequence.link import ArrowTypes, Link
from mermaid.sequence.logic import Alt, Break, Critical, Loop, Optional, Parallel


class SequenceDiagram(Graph):
    def __init__(self,
                 title: str,
                 elements: list[Union[Actor, Participant, Box, Note, Link, Alt,
                                      Break, Critical, Loop, Optional,
                                      Parallel]],
                 auto_number: bool = False):
        super().__init__(title, '')
        self.elements = elements
        self.auto_number = auto_number
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = '\nsequenceDiagram\n'
        if self.auto_number:
            script += '\tautonumber\n'
        for element in self.elements:
            script += str(element)
        self.script += script



"""UserJourney module.

This module provides the UserJourney class for creating and manipulating user journeys.
It also provides the Section and Task classes for representing sections and tasks in a user journey.

Classes:
    UserJourney: Represents a user's journey through a website or application.
    Section: Represents a section in a user's journey.
    Task: Represents a task in a section of a user's journey.
"""
from typing import Union

from mermaid.graph import Graph

from .actor import Actor
from .section import Section
from .task import Task


class UserJourney(Graph):
    """UserJourney class.

    This class represents a user's journey through a website or application.

    Attributes:
        title (str): The title of the user's journey.
        sections (list[Union[Section, Task]]): The sections in the user's journey.
    """
    def __init__(self, title: str, sections: list[Union[Section,
                                                        Task]]) -> None:
        """Initialize a new UserJourney.

        Args:
            title (str): The title of the user's journey.
            sections (list[Union[Section, Task]]): The sections in the user's journey.
        """
        super().__init__(title, '')
        self.title: str = title
        self.sections: list[Union[Section, Task]] = sections
        self._build_script()

    def _build_script(self) -> None:
        """Build the script for the user's journey."""
        super()._build_script()
        script: str = f'\njourney\n\ttitle {self.title}\n'
        for section in self.sections:
            script += f'{section}\n'
        self.script += script



from enum import Enum
from typing import Union

from .node import Node

LINK_SHAPES: dict[str, str] = {
    'normal': '--',
    'dotted': '-.-',
    'thick': '==',
    'hiden': '~~~'
}

LINK_HEADS: dict[str, str] = {
    'none': '',
    'arrow': '>',
    'left-arrow': '<',
    'bullet': 'o',
    'cross': 'x',
}


class LinkShape(Enum):
    """LinkShape Enum.

    This enum represents the different shapes a link can have in a flowchart.
    """
    NORMAL = 'normal'
    DOTTED = 'dotted'
    THICK = 'thick'
    HIDEN = 'hiden'


class LinkHead(Enum):
    """LinkHead Enum.

    This enum represents the different types of heads a link can have in a flowchart.
    """
    NONE = 'none'
    ARROW = 'arrow'
    LEFT_ARROW = 'left-arrow'
    BULLET = 'bullet'
    CROSS = 'cross'


class Link:
    """Link class.

    This class represents a link between nodes in a flowchart.

    Attributes:
        origin (Node): The origin node of the link.
        end (Node): The end node of the link.
        shape (Union[str, LinkShape]): The shape of the link.
        head_left (Union[str, LinkHead]): The head of the link at the origin node.
        head_right (Union[str, LinkHead]): The head of the link at the end node.
        message (str): The message of the link.
    """
    def __init__(self,
                 origin: Node,
                 end: Node,
                 shape: Union[str, LinkShape] = 'normal',
                 head_left: Union[str, LinkHead] = 'none',
                 head_right: Union[str, LinkHead] = 'arrow',
                 message: str = '') -> None:
        """Initialize a new Link.

        Args:
            origin (Node): The origin node of the link.
            end (Node): The end node of the link.
            shape (Union[str, LinkShape]): The shape of the link.
            head_left (Union[str, LinkHead]): The head of the link at the origin node.
            head_right (Union[str, LinkHead]): The head of the link at the end node.
            message (str): The message of the link.
        """
        self.oigin: Node = origin
        self.end: Node = end
        self.head_left: str = LINK_HEADS[
            head_left if isinstance(head_left, str) else head_left.value]
        self.head_right: str = LINK_HEADS[
            head_right if isinstance(head_right, str) else head_right.value]
        self.shape: str = LINK_SHAPES[shape if isinstance(shape, str
                                                          ) else shape.value]
        self.message: str = f'|{message}|' if message else message

    def __str__(self) -> str:
        """Return a string representation of the link.

        If the link has a message, it includes the message in the string representation.
        Otherwise, it returns a string representation of a link without a message.

        Returns:
            str: A string representation of the link.
        """
        if self.message:
            element: list[str] = [
                self.oigin.id_,
                ' ',
                self.head_left,
                self.shape,
                self.head_right,
                self.message,
                ' ',
                self.end.id_,
            ]
        else:
            element = [
                self.oigin.id_,
                ' ',
                self.head_left,
                self.shape,
                self.head_right,
                ' ',
                self.end.id_,
            ]
        return ''.join(element)



"""Mermaid module.

This module provides the Mermaid class for creating and manipulating Mermaid diagrams.
It also provides utility functions for loading data and converting text to snake case.

Classes:
    Mermaid: Represents a Mermaid diagram.

Functions:
    load(file_path): Load data from a file.
    text_to_snake_case(text): Convert a string of text to snake case.
"""
from ._main import Mermaid
from ._utils import load, text_to_snake_case

__version__: str = '0.2.5'



"""Graph module.

This module provides the Graph base class for creating and manipulating
different types of diagrams such as flowcharts, ER diagrams, etc.

Classes:
    Graph: Represents a base class for different types of diagrams.
"""
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Optional


@dataclass
class Graph:
    """Graph base class.

    This class serves as a base for other classes representing different
    types of diagrams like `Flowchart`, `ERDiagram`, etc.

    Attributes:
        title (str): The title of the diagram.
        script (str): The main script to create the diagram.
    """
    title: str
    script: str

    def save(self, path: Optional[Path] = None) -> None:
        file_path: Path = path if path else Path(f'./{self.title}.mmd')
        if file_path.suffix not in ['.mmd', '.mermaid']:
            raise ValueError("File extension must be '.mmd' or '.mermaid'")
        with open(file_path, 'w') as file:
            file.write(self.script)

    def _build_script(self) -> None:
        script: str = f'---\ntitle: {self.title}\n---' + self.script
        self.script = script



from enum import Enum
from typing import Union


class Risk(Enum):
    """Risk Enum.

    This enum represents the different levels of risk associated with a requirement in a requirement diagram.
    """
    LOW = 'Low'
    MEDIUM = 'Medium'
    HIGH = 'High'


class VerifyMethod(Enum):
    """VerifyMethod Enum.

    This enum represents the different methods of verifying a requirement in a requirement diagram.
    """
    ANALYSIS = 'Analysis'
    INSPECTION = 'Inspection'
    TEST = 'Test'
    DEMONSTRATION = 'Demonstration'


class Type(Enum):
    """Return a string representation of the link.

        Returns:
            str: A string representation of the link.
    """
    REQUIREMENT = 'requirement'
    FUNCTIONAL = 'functionalRequirement'
    INTERFACE = 'interfaceRequirement'
    PERFORMANCE = 'performanceRequirement'
    PHYSICAL = 'physicalRequirement'
    DESIGN_CONSTRAINT = 'designConstraint'


class Requirement:
    """Requirement class.

    This class represents a requirement in a requirement diagram.

    Attributes:
        id_ (str): The ID of the requirement.
        name (str): The name of the requirement.
        text (str): The text of the requirement.
        type_ (Union[str, Type]): The type of the requirement.
        risk (Union[str, Risk]): The risk of the requirement.
        verifymethod (Union[str, VerifyMethod]): The verification method of the requirement.
    """
    def __init__(self, id_: str, name: str, text: str, type_: Union[str, Type],
                 risk: Union[str, Risk],
                 verifymethod: Union[str, VerifyMethod]) -> None:
        """Initialize a new Requirement.

        Args:
            id_ (str): The ID of the requirement.
            name (str): The name of the requirement.
            text (str): The text of the requirement.
            type_ (Union[str, Type]): The type of the requirement.
            risk (Union[str, Risk]): The risk of the requirement.
            verifymethod (Union[str, VerifyMethod]): The verification method of the requirement.
        """
        self.id_: str = id_
        self.name: str = name
        self.text: str = text
        self.type_: str = type_ if isinstance(type_, str) else type_.value
        self.risk: str = risk if isinstance(risk, str) else risk.value
        self.verifymethod: str = verifymethod if isinstance(
            verifymethod, str) else verifymethod.value

    def __str__(self) -> str:
        string: str = ''
        string += f'{self.type_} {self.name} {{\n'
        string += f'\tid: {self.id_}\n'
        string += f'\ttext: {self.text}\n'
        string += f'\trisk: {self.risk}\n'
        string += f'\tverifymethod: {self.verifymethod}\n'
        string += '}\n'
        return string



"""
This module contains the PieChart class which extends the Graph class
from mermaid.graph module.

The PieChart class represents a PieChart diagram and provides methods
to build the Mermaid diagram script for the pie chart.

Classes:
    PieChart: Represents a PieChart diagram.
"""
from mermaid.graph import Graph


class PieChart(Graph):
    """
    This class represents a PieChart diagram.

    Attributes:
        data (dict[str, float]): The data for the pie chart.
        show_data (bool): Whether to show data on the pie chart.
    """
    def __init__(self,
                 title: str,
                 data: dict[str, float],
                 show_data: bool = False):
        """
        The constructor for the PieChart class.

        Parameters:
            title (str): The title of the pie chart.
            data (dict[str, float]): The data for the pie chart.
            show_data (bool): Whether to show data on the pie chart. Defaults to False.
        """
        super().__init__(title, '')
        self.data: dict[str, float] = data
        self.show_data: bool = show_data
        self._build_script()

    def _build_script(self) -> None:
        """
        Build the Mermaid diagram script for the pie chart.
        """
        super()._build_script()
        showData: str = ' showData' if self.show_data else ''
        script: str = f'\npie{showData}'

        for key, value in self.data.items():
            script += f'\n\t"{key}" : {value}'

        self.script += script + '\n'



"""ReqDiagram module.

This module provides the ReqDiagram class for creating and manipulating requirement diagrams.
It also provides utility classes for representing elements, links, and requirements in a requirement diagram.

Classes:
    ReqDiagram: Represents a requirement diagram.
    Element: Represents an element in a requirement diagram.
    Link: Represents a link between elements or requirements in a requirement diagram.
    Requirement: Represents a requirement in a requirement diagram.
"""
from mermaid.graph import Graph

from .element import *
from .link import *
from .requirement import *


class RequirementDiagram(Graph):
    """RequirementDiagram class.

    This class represents a requirement diagram, which is a type of graph.
    It contains elements, requirements, and links between them.

    Attributes:
        elements (list[Element]): The elements in the diagram.
        requirements (list[Requirement]): The requirements in the diagram.
        links (list[Link]): The links between elements and requirements in the diagram.
    """
    def __init__(self, title: str, elements: list[Element],
                 requirements: list[Requirement], links: list[Link]) -> None:
        """Initialize a new RequirementDiagram.

        Args:
            title (str): The title of the diagram.
            elements (list[Element]): The elements in the diagram.
            requirements (list[Requirement]): The requirements in the diagram.
            links (list[Link]): The links between elements and requirements in the diagram.
        """
        super().__init__(title, '')
        self.elements: list[Element] = elements if elements is not None else []
        self.requirements: list[
            Requirement] = requirements if requirements is not None else []
        self.links: list[Link] = links if links is not None else []
        self._build_script()

    def _build_script(self) -> None:
        super()._build_script()
        script: str = '\nrequirementDiagram\n'
        for element in self.elements:
            script += str(element) + '\n'
        for requirement in self.requirements:
            script += str(requirement) + '\n'

        for link in self.links:
            script += f'{link}\n'
        self.script += script
